@@ -205,6 +205,13 @@ int mosquitto_tls_set(struct mosquitto *mosq, const char *cafile, const char *ca
 
 	return MOSQ_ERR_SUCCESS;
 #else
+	UNUSED(mosq);
+	UNUSED(cafile);
+	UNUSED(capath);
+	UNUSED(certfile);
+	UNUSED(keyfile);
+	UNUSED(pw_callback);
+
 	return MOSQ_ERR_NOT_SUPPORTED;
 
 #endif
@@ -241,8 +248,12 @@ int mosquitto_tls_opts_set(struct mosquitto *mosq, int cert_reqs, const char *tl
 
 	return MOSQ_ERR_SUCCESS;
 #else
-	return MOSQ_ERR_NOT_SUPPORTED;
+	UNUSED(mosq);
+	UNUSED(cert_reqs);
+	UNUSED(tls_version);
+	UNUSED(ciphers);
 
+	return MOSQ_ERR_NOT_SUPPORTED;
 #endif
 }
 
@@ -254,6 +265,9 @@ int mosquitto_tls_insecure_set(struct mosquitto *mosq, bool value)
 	mosq->tls_insecure = value;
 	return MOSQ_ERR_SUCCESS;
 #else
+	UNUSED(mosq);
+	UNUSED(value);
+
 	return MOSQ_ERR_NOT_SUPPORTED;
 #endif
 }
@@ -373,6 +387,11 @@ int mosquitto_tls_psk_set(struct mosquitto *mosq, const char *psk, const char *i
 
 	return MOSQ_ERR_SUCCESS;
 #else
+	UNUSED(mosq);
+	UNUSED(psk);
+	UNUSED(identity);
+	UNUSED(ciphers);
+
 	return MOSQ_ERR_NOT_SUPPORTED;
 #endif
 }
@@ -382,26 +401,17 @@ int mosquitto_opts_set(struct mosquitto *mosq, enum mosq_opt_t option, void *val
 {
 	int ival;
 
-	if(!mosq || !value) return MOSQ_ERR_INVAL;
+	if(!mosq) return MOSQ_ERR_INVAL;
 
 	switch(option){
 		case MOSQ_OPT_PROTOCOL_VERSION:
+			if(value == NULL){
+				return MOSQ_ERR_INVAL;
+			}
 			ival = *((int *)value);
 			return mosquitto_int_option(mosq, option, ival);
 		case MOSQ_OPT_SSL_CTX:
-#ifdef WITH_TLS
-			mosq->ssl_ctx = (SSL_CTX *)value;
-			if(mosq->ssl_ctx){
-#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && !defined(LIBRESSL_VERSION_NUMBER)
-				SSL_CTX_up_ref(mosq->ssl_ctx);
-#else
-				CRYPTO_add(&(mosq->ssl_ctx)->references, 1, CRYPTO_LOCK_SSL_CTX);
-#endif
-			}
-			break;
-#else
-			return MOSQ_ERR_NOT_SUPPORTED;
-#endif
+			return mosquitto_void_option(mosq, option, value);
 		default:
 			return MOSQ_ERR_INVAL;
 	}
@@ -493,17 +503,17 @@ int mosquitto_int_option(struct mosquitto *mosq, enum mosq_opt_t option, int val
 
 int mosquitto_void_option(struct mosquitto *mosq, enum mosq_opt_t option, void *value)
 {
-	if(!mosq || !value) return MOSQ_ERR_INVAL;
+	if(!mosq) return MOSQ_ERR_INVAL;
 
 	switch(option){
 		case MOSQ_OPT_SSL_CTX:
 #ifdef WITH_TLS
-			mosq->ssl_ctx = (SSL_CTX *)value;
-			if(mosq->ssl_ctx){
-#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && !defined(LIBRESSL_VERSION_NUMBER)
-				SSL_CTX_up_ref(mosq->ssl_ctx);
+			mosq->user_ssl_ctx = (SSL_CTX *)value;
+			if(mosq->user_ssl_ctx){
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+				SSL_CTX_up_ref(mosq->user_ssl_ctx);
 #else
-				CRYPTO_add(&(mosq->ssl_ctx)->references, 1, CRYPTO_LOCK_SSL_CTX);
+				CRYPTO_add(&(mosq->user_ssl_ctx)->references, 1, CRYPTO_LOCK_SSL_CTX);
 #endif
 			}
 			break;
