@@ -39,7 +39,9 @@ Contributors:
 #  include <ws2tcpip.h>
 #endif
 
-#if !defined(WIN32) && !defined(__CYGWIN__) && !defined(__QNX__)
+#if defined(__HAIKU__)
+#  include <syslog.h>
+#elif !defined(WIN32) && !defined(__CYGWIN__) && !defined(__QNX__)
 #  include <sys/syslog.h>
 #endif
 
