@@ -335,7 +335,7 @@ static int listeners__start(void)
 
 	listensock_count = 0;
 
-	if(db.config->listener_count == 0){
+	if(db.config->local_only){
 		if(listeners__start_local_only()){
 			db__close();
 			if(db.config->pid_file){
