@@ -306,6 +306,7 @@ struct mosquitto__config {
 	struct mosquitto__security_options security_options;
 };
 
+
 struct mosquitto__subleaf {
 	struct mosquitto__subleaf *prev;
 	struct mosquitto__subleaf *next;
@@ -317,12 +318,6 @@ struct mosquitto__subleaf {
 };
 
 
-struct mosquitto__subshared_ref {
-	struct mosquitto__subhier *hier;
-	struct mosquitto__subshared *shared;
-};
-
-
 struct mosquitto__subshared {
 	UT_hash_handle hh;
 	char *name;
@@ -339,6 +334,12 @@ struct mosquitto__subhier {
 	uint16_t topic_len;
 };
 
+struct mosquitto__client_sub {
+	struct mosquitto__subhier *hier;
+	struct mosquitto__subshared *shared;
+	char topic_filter[];
+};
+
 struct sub__token {
 	struct sub__token *next;
 	char *topic;
