@@ -5,24 +5,15 @@ import ptest
 
 tests = [
     #(ports required, 'path'),
+    (1, './01-connect-575314.py'),
     (1, './01-connect-allow-anonymous.py'),
-    (1, './01-connect-bad-packet.py'),
-    (1, './01-connect-connack-2163.py'),
     (1, './01-connect-disconnect-v5.py'),
-    (1, './01-connect-duplicate.py'),
-    (1, './01-connect-invalid-id-0.py'),
-    (1, './01-connect-invalid-id-missing.py'),
-    (1, './01-connect-invalid-id-utf8.py'),
-    (1, './01-connect-invalid-protonum.py'),
-    (1, './01-connect-invalid-reserved.py'),
-    (1, './01-connect-success.py'),
-    (1, './01-connect-uname-invalid-utf8.py'),
-    (1, './01-connect-uname-no-flag.py'),
+    (1, './01-connect-max-connections.py'),
+    (1, './01-connect-max-keepalive.py'),
     (1, './01-connect-uname-no-password-denied.py'),
     (1, './01-connect-uname-or-anon.py'),
     (1, './01-connect-uname-password-denied-no-will.py'),
     (1, './01-connect-uname-password-denied.py'),
-    (1, './01-connect-uname-pwd-no-flag.py'),
     (2, './01-connect-zero-length-id.py'),
 
     (1, './02-shared-qos0-v5.py'),
@@ -35,9 +26,6 @@ tests = [
     (1, './02-subpub-qos0-subscription-id.py'),
     (1, './02-subpub-qos0-topic-alias-unknown.py'),
     (1, './02-subpub-qos0-topic-alias.py'),
-    (1, './02-subpub-qos0.py'),
-    (1, './02-subpub-qos1-bad-pubcomp.py'),
-    (1, './02-subpub-qos1-bad-pubrec.py'),
     (1, './02-subpub-qos1-message-expiry-retain.py'),
     (1, './02-subpub-qos1-message-expiry-will.py'),
     (1, './02-subpub-qos1-message-expiry.py'),
@@ -45,25 +33,15 @@ tests = [
     (1, './02-subpub-qos1-oversize-payload.py'),
     (1, './02-subpub-qos1.py'),
     (1, './02-subpub-qos2-1322.py'),
-    (1, './02-subpub-qos2-bad-puback-1.py'),
-    (1, './02-subpub-qos2-bad-puback-2.py'),
-    (1, './02-subpub-qos2-bad-pubcomp.py'),
     (1, './02-subpub-qos2-pubrec-error.py'),
     (1, './02-subpub-qos2-receive-maximum-1.py'),
     (1, './02-subpub-qos2-receive-maximum-2.py'),
     (1, './02-subpub-qos2.py'),
+    (1, './02-subpub-recover-subscriptions.py'),
     (1, './02-subscribe-dollar-v5.py'),
     (1, './02-subscribe-invalid-utf8.py'),
     (1, './02-subscribe-long-topic.py'),
     (1, './02-subscribe-persistence-flipflop.py'),
-    (1, './02-subscribe-qos0.py'),
-    (1, './02-subscribe-qos1.py'),
-    (1, './02-subscribe-qos2.py'),
-    (1, './02-unsubscribe-invalid-no-topic.py'),
-    (1, './02-unsubscribe-qos0.py'),
-    (1, './02-unsubscribe-qos1.py'),
-    (1, './02-unsubscribe-qos2-multiple.py'),
-    (1, './02-unsubscribe-qos2.py'),
 
     #(1, './03-publish-qos1-queued-bytes.py'),
     (1, './03-pattern-matching.py'),
@@ -106,7 +84,12 @@ tests = [
     (2, './06-bridge-br2b-disconnect-qos1.py'),
     (2, './06-bridge-br2b-disconnect-qos2.py'),
     (2, './06-bridge-br2b-remapping.py'),
-    (2, './06-bridge-clean-session.py'),
+    (2, './06-bridge-clean-session-csF-lcsF.py'),
+    (2, './06-bridge-clean-session-csF-lcsN.py'),
+    (2, './06-bridge-clean-session-csF-lcsT.py'),
+    (2, './06-bridge-clean-session-csT-lcsF.py'),
+    (2, './06-bridge-clean-session-csT-lcsN.py'),
+    (2, './06-bridge-clean-session-csT-lcsT.py'),
     (2, './06-bridge-fail-persist-resend-qos1.py'),
     (2, './06-bridge-fail-persist-resend-qos2.py'),
     (1, './06-bridge-no-local.py'),
@@ -153,7 +136,7 @@ tests = [
     (1, './09-extended-auth-multistep-reauth.py'),
     (1, './09-extended-auth-multistep.py'),
     (1, './09-extended-auth-single.py'),
-    (1, './09-extended-auth-unsupported.py'),
+    (1, './09-plugin-acl-change.py'),
     (1, './09-plugin-auth-acl-pub.py'),
     (1, './09-plugin-auth-acl-sub-denied.py'),
     (1, './09-plugin-auth-acl-sub.py'),
@@ -179,17 +162,13 @@ tests = [
 
     (1, './12-prop-assigned-client-identifier.py'),
     (1, './12-prop-maximum-packet-size-broker.py'),
-    (1, './12-prop-maximum-packet-size-connect.py'),
     (1, './12-prop-maximum-packet-size-publish-qos1.py'),
     (1, './12-prop-maximum-packet-size-publish-qos2.py'),
-    (1, './12-prop-maximum-packet-size-publish.py'),
     (1, './12-prop-response-topic-correlation-data.py'),
     (1, './12-prop-response-topic.py'),
     (1, './12-prop-server-keepalive.py'),
-    (1, './12-prop-session-expiry-invalid.py'),
     (1, './12-prop-subpub-content-type.py'),
     (1, './12-prop-subpub-payload-format.py'),
-    (1, './12-prop-topic-alias-invalid.py'),
 
     (1, './13-malformed-publish-v5.py'),
     (1, './13-malformed-subscribe-v5.py'),
